const db = require("./db.js");

const VALID_EXTENSIONS = new Set([
    "png",
    "jpg",
    "jpeg",
    "bmp",
]);

function isDevelopment() {
    return process.env.NODE_ENV == "dev";
}

function shouldShowMaintenanceMessage(serverId) {
    if(process.env.NODE_ENV != "dev")
        return false;
    
    return !db.get("devServers").includes(serverId).value();
}

function isValidImage(fileName) {
    let extPosition = fileName.lastIndexOf(".");
    if(extPosition < 0)
        return false;
    let ext = fileName.substring(extPosition + 1).toLowerCase();
    return VALID_EXTENSIONS.has(ext);
}

function isAuthorised(member) {
    if (db.get("editors.users").includes(member.id).value())
        return true;
    if (db.get("editors.roles").intersectionWith(member.roles.keyArray()).isEmpty().value())
        return false;
    return true;
}

module.exports = {
    isAuthorised: isAuthorised,
    isValidImage: isValidImage,
    shouldShowMaintenanceMessage: shouldShowMaintenanceMessage,
    isDevelopment: isDevelopment,
    documentation: {}
};